/* 
 * Criado por Bruno R. Lima em 15/03/2005
 */
package br.com.csvparser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import javax.swing.table.DefaultTableModel;

import br.com.csvconverter.CSVExcelConverter;

/**
 * Its the default reader for a CSVReader. It reads the file and load into memory ( into a
 * ArrayList more specific. Then, you can do any kind of surch into, or get a List of values
 * and maybe a TableModel.<br>
 * This class has a lot of usable methods to help you interate with your CSV file.
 * 
 * 
 * @author Bruno Rios Lima
 */
public class CSVDefaultReader implements CSVReader {

    /** contents of csv file */
    private ArrayList contents = null;
    /** Column separator. */
    public static final char COLUMN_SEPARATOR = ';';
    /** Quote separator for values */
    public static final char QUOTE_DELIMITER = '"';
    /** array of max column size */
    private Hashtable column_max_size = new Hashtable();
    /** the column separator set by user */
    public char characterColumnSeparator = COLUMN_SEPARATOR;
    /** the quote delimiter set by the user */
    public char characterQuoteDelimiter = QUOTE_DELIMITER;
    /** name */
    private String name = "";

    
    
    /**
     * Default contructor for the class CSVDefaultReader.java.
     * After create this class, you should call the method loadFile to load the CSV
     * file into memory, otherwise you will not be able to use thic clas.
     */
    public CSVDefaultReader(){
        super();
    }
    
    public CSVDefaultReader(CSVDefaultWriter writer){
        contents = (ArrayList)convertFromCSVWriter(writer.getDataInList());
    }
    
    /**
     * Contructor for the class CSVDefaultReader.java.
     * It load the file at contructor time.
     * @param file
     */
    public CSVDefaultReader(String file) throws CSVReaderException{
        // load the CSV file
        loadFile(file);
    }
    
    public CSVDefaultReader(Reader reader) throws CSVReaderException{
        // load the CSV file
        loadFile(reader);
    }
    
    /**
     * Return the values of the given row and column.
     * 
     * @param row the row to read
     * @param col the column to read
     * @return the value at given row and column
     */
    public String getVal(int row, int col){
        // return the val of the column
        return getRow(row)[col];
    }
    
    /**
     * Return the value od the given row and column, by column
     * name. It simply finds the column name using the findColumn() 
     * method.
     * 
     * @param row
     * @param columnName
     * @return
     */
    public String getVal(int row, String columnName){
        return getVal(row, findColumn(columnName));
    }
    
    /**
     * Return an String array of columns at given row.
     * The size of this array shloud be the getColumnCount value.
     * @param row the row to read the columns
     * @return an string array of the columns at this row
     */
    public String [] getRow(int row){
        // get the content of the given line and broke it into array of string
        String line = (String)contents.get(row);
        int cols = getColumnCount(row);
        String [] values = new String[cols];
        int pos = line.indexOf(characterColumnSeparator);
        int posOpenQuote = line.indexOf(characterQuoteDelimiter);
        int posCloseQuote = 0;
        boolean hasQuote = false;
        // check if starts from quote. If does, find closing quote and
        // then find the separator
        if( posOpenQuote >= 0 && posOpenQuote < pos ){
            posCloseQuote = line.indexOf(characterQuoteDelimiter, posOpenQuote+1);
            pos = line.indexOf(characterColumnSeparator, posCloseQuote+1);
            hasQuote = true;
        }
        int last_pos = 0;
        for( int col=0; col<cols; col++){
            if( hasQuote )
                values[col] = line.substring(last_pos+1,pos-1);
            else
                values[col] = line.substring(last_pos,pos);
            last_pos = pos+1;
            pos = line.indexOf(characterColumnSeparator, (last_pos+1));
            posOpenQuote = line.indexOf(characterQuoteDelimiter, last_pos);
            hasQuote = false;
            if( posOpenQuote >= 0 && posOpenQuote < pos ){
                posCloseQuote = line.indexOf(characterQuoteDelimiter, posOpenQuote+1);
                pos = line.indexOf(characterColumnSeparator, posCloseQuote+1);
                hasQuote = true;
            }
            if( pos < 0 ) pos = line.length();
        }
        return values;
    }
    
    /**
     * Return the column count of given row.
     * @param row
     * @return
     */
    private int getColumnCount(int row){
        // get the content of the given line and broke it into array of string
        String line = (String)contents.get(row);
        int pos = line.indexOf(characterColumnSeparator);
        int posOpenQuote = line.indexOf(characterQuoteDelimiter);
        int posCloseQuote = 0;
        // check if starts from quote. If does, find closing quote and
        // then find the separator
        if( posOpenQuote >= 0 && posOpenQuote < pos ){
            posCloseQuote = line.indexOf(characterQuoteDelimiter, posOpenQuote+1);
            pos = line.indexOf(characterColumnSeparator, posCloseQuote+1);
        }        
        int cols = 0;
        int last_pos = 0;
        // count the columns
        while( pos > 0 ){
            cols++;
            last_pos = pos;
            pos = line.indexOf(characterColumnSeparator, pos+1);
            posOpenQuote = line.indexOf(characterQuoteDelimiter, pos+1);
            if( posOpenQuote >= 0 && posOpenQuote < pos ){
                posCloseQuote = line.indexOf(characterQuoteDelimiter, posOpenQuote+1);
                pos = line.indexOf(characterColumnSeparator, posCloseQuote+1);
            }        
            // if the found position is not the end of the file,
            // then last column must be created by hand
            if( (last_pos+1) < line.length() && pos < 0){
                cols++;
            }
        };  
        return cols;
    }
    
    /**
     * Try to find the value in the CSV returning the number of repetitions
     * @param value
     * @return the number of occurences in the CSV file
     */
    public int findCount(String value){
        if( value == null ) return 0;
        int total = 0;
        // loop in all rows, reading all columns, trying to find this value
        for( int row=0; row<contents.size(); row++){
            for( int col=0; col<getColumnCount(); col++){
                if( value.equals(getVal(row, col)) ){
                    total++;
                }
            }
        }
        return total;
    }
    
    /**
     * Find the position XY of the occurence.
     * It return the first Row and Column that matches exactly the value in the cell.
     * @param value the value to find
     * @return an int array containing row and column in this order
     */
    public int[] find(String value){
        return find(value,1,0); // start from first line, not header
    }
    
    /**
     * Find the position XY of the occurence.
     * It return the first Row and Column that matches exactly the value in the cell
     * startinf from the given line and given column.
     * 
     * @param value
     * @param startFromRow given row to start read from
     * @param startFromColumn given column to start read from. all rows after given row
     * will start form sero again.
     * @return an int array containing row and column in this order
     */
    public int[] find(String value, int startFromRow, int startFromColumn){
        if( value == null ) return null;
        int columnToStart = startFromColumn;
        int [] positionXY = new int[2];
        // loop in all rows, reading all columns, trying to find this value
        for( int row=startFromRow; row<contents.size(); row++){
            // if passed in the start row, reads all columns again
            if( row > startFromRow ) columnToStart = 0; 
            for( int col=columnToStart; col<getColumnCount(); col++){
                if( value.equals(getVal(row, col)) ){
                    // found the val, what to do?
                    positionXY[0] = row; // X (row)
                    positionXY[1] = col; // Y (col)
                    return positionXY;
                }
            }
        }
        return null;
    }
    
    /**
     * looks in all rows in this specific column, returning the rows this value is.
     * @param value the value to find
     * @param col the column to look for it
     * @return an ArrayList containing the rows found ( Integer )
     */
    public ArrayList findByColumn(String value, int col){
        if( value == null ) return null;
        int [] positionXY = new int[2];
        ArrayList founds = new ArrayList();
        // loop in all rows, trying to find this value
        for( int row=1; row<contents.size(); row++){
            if( value.equals(getVal(row, col)) ){
                // found the val, what to do?
                founds.add(new Integer(row));
            }
        }
        return founds;
    }

    /**
     * looks in all columns in this specific row, returning the columns this value is.
     * @param value the value to find
     * @param row the row to look for it
     * @return an ArrayList containing the columns found ( Integer )
     */
    public ArrayList findByRow(String value, int row){
        if( value == null ) return null;
        int [] positionXY = new int[2];
        ArrayList founds = new ArrayList();
        // loop in all rows, trying to find this value
        for( int col=0; col<getColumnCount(); col++){
            if( value.equals(getVal(row, col)) ){
                // found the val, what to do?
                founds.add(new Integer(row));
            }
        }
        return founds;
    }
    
    /**
     * Return a DefaultTableModel with the data
     * @return
     */
    public DefaultTableModel getDataInTableModel(){
        return getDataInTableModel(true);
    }
    
    /**
     * Return a DefaultTableModel with the data
     * @return
     * @param ignoreFirstRow ignore the first row
     */
    public DefaultTableModel getDataInTableModel(boolean ignoreFirstRow){
        // create the tablemodel
        DefaultTableModel m = new DefaultTableModel();
        // read the columns
        String [] columns = getRow(0);
        // add columns to the tablemodel
        for( int col=0; col<columns.length; col++)
            m.addColumn(columns[col]);
        // add rows ( data )
        for( int row=(ignoreFirstRow?1:0); row<getRowCount(); row++)
            m.addRow(getRow(row));
        // return the model
        return m;
    }
    
    /**
     * Return the List object with the lines of this file.<br>
     * Ex:<br> 
     * <code>
     * int row = 4;<br>
     * String row_value = (String)getDataInList().get(row);<br>
     * </code><br>
     * It should bring the entire line just as it is in the CSV file.
     * 
     * @return
     */
    public List getDataInList(){
        return contents;
    }
    
    /**
     * Return the row count read when loading the file.
     * @return the row count
     */
    public int getRowCount(){
        return contents.size();
    }
    
    /**
     * Return the column count. It reads the header ( row zero ) and counts the columns
     * separeted by the characterColumnSeparator.
     * @return the number of columns
     */
    public int getColumnCount(){
        return getRow(0).length;
    }
    
    /**
     * Try to open a File with this address and call loadFile(File) to laod it.
     * @param file String that contains the address of the file.
     */
    public void loadFile(String file) throws CSVReaderException{
        if(file == null) return;
        loadFile(new File(file));
    }
    
    /**
     * Load the given File into the memory.
     * @param file the File to read.
     */
    public void loadFile(File file) throws CSVReaderException{
        if(file == null) return;
        try{
            loadFile(new FileReader(file));
        } catch (FileNotFoundException ex) {
            throw new CSVReaderException(ex);
        }
    }

    /**
     * Use the reader to read the file content.
     * It reads the file as BufferedReader and, line to line, put this line as
     * String into the array contents.
     * @param reader
     * @throws CSVReaderException
     */
    public void loadFile(Reader reader) throws CSVReaderException{
        
        if(reader == null) return;
        contents = new ArrayList();
        //declared here only to make visible to finally clause
        BufferedReader input = null;
        try {
            //use buffering
            //this implementation reads one line at a time
            input = new BufferedReader(reader);
            String line = null; //not declared within while loop
            int headerColumnCount = 0;
            while ((line = input.readLine()) != null) {
                contents.add(line);
                // try to find the maximum size of each column and keep it
                // store to the current line
                storeMaxColumnSize(contents.size()-1);
                // check if the current row has the same column size of header
                /*if( (contents.size()-1) == 0 ){
                    headerColumnCount = getColumnCount((contents.size()-1));
                }else if( (contents.size()-1) > 0 ){
                    if( getColumnCount((contents.size()-1)) > headerColumnCount ){
                        CSVReaderException e = new CSVReaderException(
                                "The row "+ (contents.size()-1) +
                                " has more columns ("+getColumnCount((contents.size()-1))+
                                ") than header("+headerColumnCount+").");
                        e.row = (contents.size()-1);
                        throw e;
                    }
                }*/
                //contents.append(System.getProperty("line.separator"));
            }
        } catch (FileNotFoundException ex) {
            throw new CSVReaderException(ex);
        } catch (IOException ex) {
            throw new CSVReaderException(ex);
        } finally {
            try {
                if (input != null) {
                    //flush and close both "input" and its underlying
                    // FileReader
                    input.close();
                }
            } catch (IOException ex) {
                throw new CSVReaderException(ex);
            }
        }        
        
    }
    
    /**
     * Return the toString of ArrayList of contents.
     */
    public String toString(){
        return contents.toString();
    }
    
    public String getDataInString(){
        StringBuffer b = new StringBuffer("");
        for( int row=0; row<contents.size(); row++){
            String[] line = getRow(row);
            for(int col=0; col<line.length;col++){
                b.append(characterQuoteDelimiter);
                b.append(line[col]);
                b.append(characterQuoteDelimiter);
                b.append(characterColumnSeparator);
            }
            b.append(System.getProperty("line.separator"));
        }
        return b.toString();
    }
    
    /**
     * Simply print the contents of the CSV file into an out stream.
     */
    public void printContents(){
        for( int row=0; row<contents.size(); row++){
            printContents(row);
        }
    }
    
    /**
     * Simply print the contents of the CSV file into default out stream.
     * @param row
     */
    public void printContents(int row){
        String[] line = getRow(row);
        System.out.print(row + ") ");
        for(int col=0; col<line.length;col++){
            System.out.print(" [" + line[col] + "]");
        }
        System.out.print(System.getProperty("line.separator"));
    }
    
    /**
     * Read the columns of the given row and store the size in a hashtable
     * if the current column length is greater then the old one.
     * @param row the row to analize the columns.
     */
    private void storeMaxColumnSize(int row){
        String [] cols = getRow(row);
        for(int col=0; col<cols.length;col++){
            if( cols[col] == null )
                cols[col] = "";
            //to each col store the size if it is greater then the stored
            Integer old_size = (Integer)column_max_size.get(new Integer(col));
            if( old_size == null || old_size.intValue() <= cols[col].length() ){
                // store new val
                column_max_size.put(new Integer(col), new Integer(cols[col].length()));
            }
        }
    }
    
    /**
     * Return the maximum size of this column of all rows.
     * Can be used to format texts.
     * @param col the column desired to know the max size
     * @return the size
     */
    public int getColumnMaxSize(int col){
        return ((Integer)column_max_size.get(new Integer(col))).intValue();
    }

    /**
     * Return the column separator set bby user
     * @return
     */
    public char getCharacterColumnSeparator() {
        return characterColumnSeparator;
    }
    
    /**
     * Get the column name from the header
     * @param col the column do get the name
     * @return the name as string
     */
    public String getColumnName(int col){
        String[] header = getRow(0);
        return header[col].toUpperCase();
    }
    
    /**
     * Try to find the column name and return the index for that column in the CSV file.
     * Does not matter the case sensitive.
     * @param columnName the column name
     * @return the index of this column in the file
     */
    public int findColumn(String columnName){
        String[] header = getRow(0);
        for( int c=0; c<header.length; c++)
            if(header[c].toUpperCase().equals(columnName.toUpperCase()))
                return c;
        return -1;
    }
    
    private List convertFromCSVWriter(List ct){
        // what should be done:
        // it comes like list(array) -> list(string)
        ArrayList outLs = new ArrayList();
        ArrayList wrCon = (ArrayList)ct;
        StringBuffer strLine = new StringBuffer("");
        for( int row=0; row<wrCon.size(); row++){
            ArrayList ln = (ArrayList)wrCon.get(row);
            for( int col=0; col<ln.size(); col++){
                strLine.append(characterQuoteDelimiter);
                strLine.append((String)ln.get(col));
                strLine.append(characterQuoteDelimiter);
                strLine.append(characterColumnSeparator);
            }
            // change line
            outLs.add(strLine.toString());
            strLine.delete(0,strLine.length());
        }
        return outLs;
    }
    
    public String getName() {
        if( name == null || name.length() == 0 )
            return "CSVReader";
        return name;
    }
    public void setName(String name) {
        this.name = name;
    }
}
