/* 
 * Criado por Bruno R. Lima em 16/03/2005
 */
package br.com.csvparser;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

import javax.swing.table.TableModel;

/**
 * A Default writer for CSV files. It writes CSV files. You can write files just
 * by insertVal(String) and save(), and setting the file name and path to save it.
 * 
 * @author Bruno Rios Lima
 */
/**
 * 
 * @author Bruno Rios Lima
 */
public class CSVDefaultWriter implements CSVWriter {
    
    private ArrayList contents = new ArrayList();
    private String fileName = "emptyCSVFile.csv";
    private Writer fileWriter = null;
    
    /** Column separator. */
    public static final char COLUMN_SEPARATOR = ';';
    /** Quote separator for values */
    public static final char QUOTE_DELIMITER = '"';
    /** the column separator set by user */
    public char characterColumnSeparator = COLUMN_SEPARATOR;
    /** the quote delimiter set by the user */
    public char characterQuoteDelimiter = QUOTE_DELIMITER;
    
    
    public static void main(String args[]){
        
        // open the writer
        CSVDefaultWriter w = new CSVDefaultWriter("SRC/teste.CSV");
        
        // insert data
        //---------------------------------
        
        System.gc();
        
        
        
        
//      -------------------------------------------------------------------------------
        
        /*w.insertRow(new String[]{"COL1","COL2","COL3","COL4"});
        w.insertRow(new String[]{"COL1","COL2","COL3","COL4"});
        w.insertRow(new String[]{"COL1","COL2","COL3","COL4"});
        w.insertRow(new String[]{"COL1","COL2","COL3","COL4"});
        w.insertRow(new String[]{"COL1","COL2","COL3","COL4"});
        w.insertVal("Coluna suja");
        
        w.setHeader(new String[]{"COLUNA1","COLUNA2","COLUNA3","COLUNA4"});
        
        
        CSVDefaultReader c = new CSVDefaultReader(w);
        c.printContents();
        
        //---------------------------------
        // save into the file ( phisical )
        w.save();*/
        
        
        //-------------------------------------------------------------------------------
        
        
        /*CSVDefaultReader c = null;
        
        try {
            c = new CSVDefaultReader();
            // Open , read and close the file. Can be temporary if its the case.
            c.loadFile("src/emptyCSVFile.csv");
            c.printContents();
        } catch (CSVReaderException e) {
            e.printStackTrace();
        }*/
        
        
    }
    
    /**
     * Default contructor for the class CSVDefaultWriter.java.
     */
    public CSVDefaultWriter(){
        super();
    }
    
    /**
     * Contrutor that receive the file name.
     * @param fileName
     */
    public CSVDefaultWriter(String fileName){
        if( fileName != null )
            this.fileName = fileName;
    }
    
    public CSVDefaultWriter(Writer writer){
        if( fileWriter != null )
            this.fileWriter = writer;
    }
    
    public CSVDefaultWriter(TableModel model, String fileName){
        if( fileName != null )
            this.fileName = fileName;
        loadDataFromTableModel(model);
    }
    
    /**
     * Load data from TableModel, add data to the buffer.
     * If there is data into the buffer, it will add more data.
     * If you want to start from a fresh onw, call clear() first.
     * 
     * @param model
     */
    public void loadDataFromTableModel(TableModel model){
        // create the header ( column names )
        String[] header = new String[model.getColumnCount()];
        for( int col=0; col<model.getColumnCount(); col++){
            header[col] = model.getColumnName(col); 
        }
        setHeader(header);
        // feed the data onto memory
        for( int row=0; row<model.getRowCount(); row++){
            insertRow(null);
            for( int col=0; col<model.getColumnCount(); col++){
                insertVal(model.getValueAt(row, col)==null?"":model.getValueAt(row, col).toString());
            }
        }
    }
    
    public CSVDefaultWriter(CSVDefaultReader reader, String fileName){
        if( fileName != null )
            this.fileName = fileName;
        contents = (ArrayList)convertFromCsvReader(reader);
    }

    private List convertFromCsvReader(CSVDefaultReader reader){
        // what should be done:
        // it comes like list(String) -> list(Array)
        ArrayList out = new ArrayList();
        ArrayList ln = null;
        for( int row=0; row<reader.getRowCount(); row++){
            String[] line = (String[])reader.getRow(row);
            ln = new ArrayList();
            for(int col=0; col<line.length;col++){
                ln.add(line[col]);
            }
            out.add(ln);
        }
        return out;
    }
    
    public void loadDataFromCSVReader(CSVDefaultReader reader){
        contents = (ArrayList)convertFromCsvReader(reader);
    }
    
    /**
     * Insert a entire row at CSV file
     */
    public void insertRow(String[] values) {
        if( values == null ){
            // ad new line to the contents with no columns
            ArrayList ln = new ArrayList();
            contents.add(ln);
        }else{
            // converts the current array into arraylist
            // and add it into contents
            ArrayList ln = new ArrayList();
            for( int col=0; col<values.length; col++){
                ln.add(values[col]);
            }
            // add the converted array to contents
            contents.add(ln);
        }
    }

    /**
     * Insert values to the next row
     * @param value
     */
    public void insertVal(String [] value) {
        // converts the current array into arraylist
        // and add it into contents
        ArrayList ln = (ArrayList)contents.get(getRowCount()-1);
        for( int col=0; col<value.length; col++){
            ln.add(value[col]);
        }
    }
    
    /**
     * Insert a value at the end.
     * @param value
     */
    public void insertVal(String value) {
        // insert this value at last row
        insertVal(value, getRowCount()-1);
    }
    
    /**
     * Insert a values at the desired row
     * @param value
     * @param row
     */
    public void insertVal(String [] value, int row) {
        ArrayList ln = (ArrayList)contents.get(row);
        for( int col=0; col<value.length; col++){
            ln.add(value[col]);
        }
    }
    
    /**
     * Insert a unique value at the desired row
     */
    public void insertVal(String value, int row) {
        // get the given row and add a column
        ArrayList ln = (ArrayList)contents.get(row);
        ln.add(value);
    }

    /**
     * Update the entire desired row
     */
    public void updateRow(String[] values, int row) {
        // converts the current array into arraylist
        // and add it into contents
        ArrayList ln = new ArrayList();
        for( int col=0; col<values.length; col++){
            ln.add(values[col]);
        }
        // add the converted array to contents
        contents.set(row, ln);
    }

    /**
     * Update the value at desired row and column
     */
    public void updateVal(String value, int row, int col) {
        ArrayList ln = (ArrayList)contents.get(row);
        ln.set(col, value);
    }

    /**
     * Save the buffer into the file.
     * It simply write the file
     */
    public void save(){
        try {
            if( fileName == null || fileName.length() == 0 ){
                save(null);
            }else{
                save(new File(fileName));
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Save the given file ( created previously ) phisically
     * @param aFile
     * @throws FileNotFoundException
     * @throws IOException
     */
    public void save(File aFile) throws FileNotFoundException, IOException {

        if( fileWriter == null ){
        
	        if (aFile == null) {
	            throw new IllegalArgumentException("File should not be null.");
	        }
	        if (!aFile.exists()) {
	            //throw new FileNotFoundException("File does not exist: " + aFile);
	            aFile.createNewFile();
	        }
	        if (!aFile.isFile()) {
	            throw new IllegalArgumentException("Should not be a directory: "
	                    + aFile);
	        }
	        if (!aFile.canWrite()) {
	            throw new IllegalArgumentException("File cannot be written: "
	                    + aFile);
	        }
	        
        }

        //declared here only to make visible to finally clause; generic
        // reference
        Writer output = null;
        try {
            //use buffering
            if( fileWriter != null ){
                output = fileWriter;
            }else{
                output = new BufferedWriter(new FileWriter(aFile));
            }            
            
            //output.write( aContents );
            // write all contents of contents into file
            for( int row=0; row<getRowCount(); row++){
                ArrayList ln = (ArrayList)contents.get(row);
                for( int col=0; col<ln.size(); col++){
                    // print the value
                    output.write( 
                            characterQuoteDelimiter + 
                            (String)ln.get(col) + 
                            characterQuoteDelimiter + 
                            characterColumnSeparator );
                }
                // change line
                output.write(System.getProperty("line.separator"));
            }
            
        } finally {
            //flush and close both "output" and its underlying FileWriter
            if (output != null)
                output.close();
        }
        //System.out.println("Sucessfully saved file ["+ aFile.getName() +"]!");
    }

    /**
     * Set the header of the file. Usually its the part that contains 
     * the column names.
     * At CSVReader, this header can limitate other rows size based on it size.
     */
    public void setHeader(String[] values) {
        if( getRowCount() == 0 ){ // No lines
            insertRow(null);
        }
        // set the header
        updateRow(values, 0);
    }

    /**
     * Return the number of rows that exist. 
     */
    public int getRowCount() {
        return contents.size();
    }
    
    /**
     * Clear all contents of CSV buffer.
     */
    public void clear(){
        contents = new ArrayList();
    }
    
    /**
     * Return the List object with the lines of this file.<br>
     * Ex:<br> 
     * <code>
     * int row = 4;<br>
     * String row_value = (String)getDataInList().get(row);<br>
     * </code><br>
     * It should bring the entire line just as it is in the CSV file.
     * 
     * @return
     */
    public List getDataInList(){
        return contents;
    }
    
}
