/* 
 * Criado por Bruno R. Lima em 23/03/2005
 */
package br.com.csvviewer;

import java.awt.BorderLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.DefaultTableModel;

import br.com.csvconverter.CSVExcelConverter;
import br.com.csvparser.CSVDefaultReader;
import br.com.csvparser.CSVDefaultWriter;
import br.com.csvparser.CSVReaderException;

/**
 * 
 * @author Bruno Rios Lima
 */
public class CSVPaneFileViewer extends JPanel {

    public static int GRID_VIEWER = 1;
    public static int TEXT_VIEWER = 2;
    
    private JTable tba;
    private JTextArea txa;
    private JScrollPane pne;
    private int typeViewer = TEXT_VIEWER;
    private CSVDefaultReader reader = null;
    
    public String fileName; 
    
    /**
     * Default contructor. Creates a viewer with new File.
     */
    public CSVPaneFileViewer(int type){
        super();
        typeViewer = type;
        if( typeViewer == TEXT_VIEWER ){
            initTextViewer();
        }else{
            initGridViewer();
        }
        initi();
    }
    
    private void initi(){
        setLayout(new BorderLayout());
        add(pne, BorderLayout.CENTER);
    }
    
    private void initTextViewer(){
        txa = new JTextArea();
        pne = new JScrollPane(txa);
    }
    
    private void initGridViewer(){
        tba = new JTable();
        pne = new JScrollPane(tba);
        
        tba.addKeyListener(new KeyAdapter(){
            public void keyPressed(KeyEvent e){
                if( e.getKeyCode() == KeyEvent.VK_DELETE ){
                	// remove row
                    removeSelRow();
                }
                if( e.getKeyCode() == KeyEvent.VK_INSERT ){
                    //ad new line
                    insertNewRow();
                }
            }
        });
        
    }
    
    public void removeSelRow(){
        int [] rows = tba.getSelectedRows();
        DefaultTableModel m = (DefaultTableModel)tba.getModel();
        for( int r=(rows.length-1); r>=0; r-- ){
            m.removeRow(rows[r]);
        }
    }
    
    private void insertNewRow(){
        DefaultTableModel m = (DefaultTableModel)tba.getModel();
        m.addRow(new String[m.getColumnCount()]);
    }
    
    public void exportToCSV(File fileName) throws CSVViewerException{
        
        CSVDefaultWriter w = new CSVDefaultWriter();
        w.loadDataFromTableModel(tba.getModel());
        reader = new CSVDefaultReader(w);
        
        try{
            w.save(fileName);
        }catch(Exception e){
            throw new CSVViewerException(e);
        }
        loadData(reader);
    }
    
    public void exportToExcel(File fileName) throws CSVViewerException{
        
        CSVDefaultWriter w = new CSVDefaultWriter();
        w.loadDataFromTableModel(tba.getModel());
        reader = new CSVDefaultReader(w);
        
        try{
            new CSVExcelConverter().saveAsExcelFile(
                reader, new FileOutputStream(fileName));
        }catch(Exception e){
            throw new CSVViewerException(e);
        }
        loadData(reader);
    }
    
    private void loadData(CSVDefaultReader r){
        if( typeViewer == GRID_VIEWER ){
            tba.setModel(reader.getDataInTableModel());
        }else if( typeViewer == TEXT_VIEWER ){
            txa.setText(reader.getDataInString());
        }
    }
    
    public void openExcelFile(File fileName) throws CSVViewerException{
        
        try{
	        CSVExcelConverter cs = new CSVExcelConverter();
	        reader = cs.getCSVReaderFromExcel(new FileInputStream(fileName));
	        loadData(reader);
        } catch (Exception e) {
            e.printStackTrace();
            throw new CSVViewerException(e);
        }
        
    }
    
    public void open(File fileName) throws CSVViewerException{
        
        try {
            
            // tryies if its an excel file
            if( fileName.getName().toUpperCase().endsWith("XLS") ){
                openExcelFile(fileName);
                return;
            }
            
            reader = new CSVDefaultReader();
            // Open , read and close the file. Can be temporary if its the case.
            reader.loadFile(fileName);
            loadData(reader);
                
        } catch (CSVReaderException e) {
            throw new CSVViewerException(e);
        }
        
    }
    
    public CSVDefaultReader getReader(){
        return reader;
    }
    
    public String getFileName() {
        return fileName;
    }
    public void setFileName(String fileName) {
        this.fileName = fileName;
    }
    
}
